from . import sampling

def append_image_geometry_z_slice(data_container_array, input_attribute_matrix = None, destination_attribute_matrix = None, check_resolution = None, observer = None) -> int:
  """
  AppendImageGeometryZSlice

  :param data_container_array | DataContainerArray
  :param input_attribute_matrix | DataArrayPath InputAttributeMatrix
  :param destination_attribute_matrix | DataArrayPath DestinationAttributeMatrix
  :param check_resolution | bool CheckResolution
  :param observer | Observer
  """

  filter = sampling.AppendImageGeometryZSlice()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if input_attribute_matrix is not None:
    filter.InputAttributeMatrix = input_attribute_matrix
  if destination_attribute_matrix is not None:
    filter.DestinationAttributeMatrix = destination_attribute_matrix
  if check_resolution is not None:
    filter.CheckResolution = check_resolution
  filter.execute()
  return filter.ErrorCode

def resample_image_geom(data_container_array, new_data_container_path = None, cell_attribute_matrix_path = None, cell_feature_attribute_matrix_path = None, spacing = None, renumber_features = None, save_as_new_data_container = None, feature_ids_array_path = None, observer = None) -> int:
  """
  ResampleImageGeom

  :param data_container_array | DataContainerArray
  :param new_data_container_path | DataArrayPath NewDataContainerPath
  :param cell_attribute_matrix_path | DataArrayPath CellAttributeMatrixPath
  :param cell_feature_attribute_matrix_path | DataArrayPath CellFeatureAttributeMatrixPath
  :param spacing | FloatVec3Type Spacing
  :param renumber_features | bool RenumberFeatures
  :param save_as_new_data_container | bool SaveAsNewDataContainer
  :param feature_ids_array_path | DataArrayPath FeatureIdsArrayPath
  :param observer | Observer
  """

  filter = sampling.ResampleImageGeom()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if new_data_container_path is not None:
    filter.NewDataContainerPath = new_data_container_path
  if cell_attribute_matrix_path is not None:
    filter.CellAttributeMatrixPath = cell_attribute_matrix_path
  if cell_feature_attribute_matrix_path is not None:
    filter.CellFeatureAttributeMatrixPath = cell_feature_attribute_matrix_path
  if spacing is not None:
    filter.Spacing = spacing
  if renumber_features is not None:
    filter.RenumberFeatures = renumber_features
  if save_as_new_data_container is not None:
    filter.SaveAsNewDataContainer = save_as_new_data_container
  if feature_ids_array_path is not None:
    filter.FeatureIdsArrayPath = feature_ids_array_path
  filter.execute()
  return filter.ErrorCode

def crop_image_geometry(data_container_array, new_data_container_name = None, cell_attribute_matrix_path = None, cell_feature_attribute_matrix_path = None, x_min = None, y_min = None, z_min = None, x_max = None, y_max = None, z_max = None, renumber_features = None, save_as_new_data_container = None, update_origin = None, feature_ids_array_path = None, observer = None) -> int:
  """
  CropImageGeometry

  :param data_container_array | DataContainerArray
  :param new_data_container_name | DataArrayPath NewDataContainerName
  :param cell_attribute_matrix_path | DataArrayPath CellAttributeMatrixPath
  :param cell_feature_attribute_matrix_path | DataArrayPath CellFeatureAttributeMatrixPath
  :param x_min | int XMin
  :param y_min | int YMin
  :param z_min | int ZMin
  :param x_max | int XMax
  :param y_max | int YMax
  :param z_max | int ZMax
  :param renumber_features | bool RenumberFeatures
  :param save_as_new_data_container | bool SaveAsNewDataContainer
  :param update_origin | bool UpdateOrigin
  :param feature_ids_array_path | DataArrayPath FeatureIdsArrayPath
  :param observer | Observer
  """

  filter = sampling.CropImageGeometry()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if new_data_container_name is not None:
    filter.NewDataContainerName = new_data_container_name
  if cell_attribute_matrix_path is not None:
    filter.CellAttributeMatrixPath = cell_attribute_matrix_path
  if cell_feature_attribute_matrix_path is not None:
    filter.CellFeatureAttributeMatrixPath = cell_feature_attribute_matrix_path
  if x_min is not None:
    filter.XMin = x_min
  if y_min is not None:
    filter.YMin = y_min
  if z_min is not None:
    filter.ZMin = z_min
  if x_max is not None:
    filter.XMax = x_max
  if y_max is not None:
    filter.YMax = y_max
  if z_max is not None:
    filter.ZMax = z_max
  if renumber_features is not None:
    filter.RenumberFeatures = renumber_features
  if save_as_new_data_container is not None:
    filter.SaveAsNewDataContainer = save_as_new_data_container
  if update_origin is not None:
    filter.UpdateOrigin = update_origin
  if feature_ids_array_path is not None:
    filter.FeatureIdsArrayPath = feature_ids_array_path
  filter.execute()
  return filter.ErrorCode

def extract_flagged_features(data_container_array, feature_ids_array_path = None, flagged_features_array_path = None, observer = None) -> int:
  """
  ExtractFlaggedFeatures

  :param data_container_array | DataContainerArray
  :param feature_ids_array_path | DataArrayPath FeatureIdsArrayPath
  :param flagged_features_array_path | DataArrayPath FlaggedFeaturesArrayPath
  :param observer | Observer
  """

  filter = sampling.ExtractFlaggedFeatures()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if feature_ids_array_path is not None:
    filter.FeatureIdsArrayPath = feature_ids_array_path
  if flagged_features_array_path is not None:
    filter.FlaggedFeaturesArrayPath = flagged_features_array_path
  filter.execute()
  return filter.ErrorCode

def nearest_point_fuse_regular_grids(data_container_array, reference_cell_attribute_matrix_path = None, sampling_cell_attribute_matrix_path = None, observer = None) -> int:
  """
  NearestPointFuseRegularGrids

  :param data_container_array | DataContainerArray
  :param reference_cell_attribute_matrix_path | DataArrayPath ReferenceCellAttributeMatrixPath
  :param sampling_cell_attribute_matrix_path | DataArrayPath SamplingCellAttributeMatrixPath
  :param observer | Observer
  """

  filter = sampling.NearestPointFuseRegularGrids()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if reference_cell_attribute_matrix_path is not None:
    filter.ReferenceCellAttributeMatrixPath = reference_cell_attribute_matrix_path
  if sampling_cell_attribute_matrix_path is not None:
    filter.SamplingCellAttributeMatrixPath = sampling_cell_attribute_matrix_path
  filter.execute()
  return filter.ErrorCode

def regularize_z_spacing(data_container_array, cell_attribute_matrix_path = None, input_file = None, new_z_res = None, observer = None) -> int:
  """
  RegularizeZSpacing

  :param data_container_array | DataContainerArray
  :param cell_attribute_matrix_path | DataArrayPath CellAttributeMatrixPath
  :param input_file | QString InputFile
  :param new_z_res | float NewZRes
  :param observer | Observer
  """

  filter = sampling.RegularizeZSpacing()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if cell_attribute_matrix_path is not None:
    filter.CellAttributeMatrixPath = cell_attribute_matrix_path
  if input_file is not None:
    filter.InputFile = input_file
  if new_z_res is not None:
    filter.NewZRes = new_z_res
  filter.execute()
  return filter.ErrorCode

def warp_regular_grid(data_container_array, new_data_container_name = None, cell_attribute_matrix_path = None, poly_order = None, second_order_a_coeff = None, second_order_b_coeff = None, third_order_a_coeff = None, third_order_b_coeff = None, fourth_order_a_coeff = None, fourth_order_b_coeff = None, save_as_new_data_container = None, observer = None) -> int:
  """
  WarpRegularGrid

  :param data_container_array | DataContainerArray
  :param new_data_container_name | DataArrayPath NewDataContainerName
  :param cell_attribute_matrix_path | DataArrayPath CellAttributeMatrixPath
  :param poly_order | int PolyOrder
  :param second_order_a_coeff | Float2ndOrderPolynomial SecondOrderACoeff
  :param second_order_b_coeff | Float2ndOrderPolynomial SecondOrderBCoeff
  :param third_order_a_coeff | Float3rdOrderPoly_t ThirdOrderACoeff
  :param third_order_b_coeff | Float3rdOrderPoly_t ThirdOrderBCoeff
  :param fourth_order_a_coeff | Float4thOrderPolynomial FourthOrderACoeff
  :param fourth_order_b_coeff | Float4thOrderPolynomial FourthOrderBCoeff
  :param save_as_new_data_container | bool SaveAsNewDataContainer
  :param observer | Observer
  """

  filter = sampling.WarpRegularGrid()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if new_data_container_name is not None:
    filter.NewDataContainerName = new_data_container_name
  if cell_attribute_matrix_path is not None:
    filter.CellAttributeMatrixPath = cell_attribute_matrix_path
  if poly_order is not None:
    filter.PolyOrder = poly_order
  if second_order_a_coeff is not None:
    filter.SecondOrderACoeff = second_order_a_coeff
  if second_order_b_coeff is not None:
    filter.SecondOrderBCoeff = second_order_b_coeff
  if third_order_a_coeff is not None:
    filter.ThirdOrderACoeff = third_order_a_coeff
  if third_order_b_coeff is not None:
    filter.ThirdOrderBCoeff = third_order_b_coeff
  if fourth_order_a_coeff is not None:
    filter.FourthOrderACoeff = fourth_order_a_coeff
  if fourth_order_b_coeff is not None:
    filter.FourthOrderBCoeff = fourth_order_b_coeff
  if save_as_new_data_container is not None:
    filter.SaveAsNewDataContainer = save_as_new_data_container
  filter.execute()
  return filter.ErrorCode

def resample_rect_grid_to_image_geom(data_container_array, rectilinear_grid_path = None, rect_grid_geometry_desc = None, selected_data_array_paths = None, image_geometry_path = None, image_geom_cell_attribute_matrix = None, length_unit = None, dimensions = None, observer = None) -> int:
  """
  ResampleRectGridToImageGeom

  :param data_container_array | DataContainerArray
  :param rectilinear_grid_path | DataArrayPath RectilinearGridPath
  :param rect_grid_geometry_desc | QString RectGridGeometryDesc
  :param selected_data_array_paths | std::vector<DataArrayPath> SelectedDataArrayPaths
  :param image_geometry_path | DataArrayPath ImageGeometryPath
  :param image_geom_cell_attribute_matrix | QString ImageGeomCellAttributeMatrix
  :param length_unit | int32_t LengthUnit
  :param dimensions | IntVec3Type Dimensions
  :param observer | Observer
  """

  filter = sampling.ResampleRectGridToImageGeom()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if rectilinear_grid_path is not None:
    filter.RectilinearGridPath = rectilinear_grid_path
  if rect_grid_geometry_desc is not None:
    filter.RectGridGeometryDesc = rect_grid_geometry_desc
  if selected_data_array_paths is not None:
    filter.SelectedDataArrayPaths = selected_data_array_paths
  if image_geometry_path is not None:
    filter.ImageGeometryPath = image_geometry_path
  if image_geom_cell_attribute_matrix is not None:
    filter.ImageGeomCellAttributeMatrix = image_geom_cell_attribute_matrix
  if length_unit is not None:
    filter.LengthUnit = length_unit
  if dimensions is not None:
    filter.Dimensions = dimensions
  filter.execute()
  return filter.ErrorCode

def sample_surface_mesh(data_container_array, surface_mesh_face_labels_array_path = None, observer = None) -> int:
  """
  SampleSurfaceMesh

  :param data_container_array | DataContainerArray
  :param surface_mesh_face_labels_array_path | DataArrayPath SurfaceMeshFaceLabelsArrayPath
  :param observer | Observer
  """

  filter = sampling.SampleSurfaceMesh()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if surface_mesh_face_labels_array_path is not None:
    filter.SurfaceMeshFaceLabelsArrayPath = surface_mesh_face_labels_array_path
  filter.execute()
  return filter.ErrorCode

def regular_grid_sample_surface_mesh(data_container_array, data_container_name = None, cell_attribute_matrix_name = None, length_unit = None, dimensions = None, spacing = None, origin = None, feature_ids_array_name = None, observer = None) -> int:
  """
  RegularGridSampleSurfaceMesh

  :param data_container_array | DataContainerArray
  :param data_container_name | DataArrayPath DataContainerName
  :param cell_attribute_matrix_name | QString CellAttributeMatrixName
  :param length_unit | int32_t LengthUnit
  :param dimensions | IntVec3Type Dimensions
  :param spacing | FloatVec3Type Spacing
  :param origin | FloatVec3Type Origin
  :param feature_ids_array_name | QString FeatureIdsArrayName
  :param observer | Observer
  """

  filter = sampling.RegularGridSampleSurfaceMesh()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if data_container_name is not None:
    filter.DataContainerName = data_container_name
  if cell_attribute_matrix_name is not None:
    filter.CellAttributeMatrixName = cell_attribute_matrix_name
  if length_unit is not None:
    filter.LengthUnit = length_unit
  if dimensions is not None:
    filter.Dimensions = dimensions
  if spacing is not None:
    filter.Spacing = spacing
  if origin is not None:
    filter.Origin = origin
  if feature_ids_array_name is not None:
    filter.FeatureIdsArrayName = feature_ids_array_name
  filter.execute()
  return filter.ErrorCode

def sample_surface_mesh_specified_points(data_container_array, input_file_path = None, output_file_path = None, observer = None) -> int:
  """
  SampleSurfaceMeshSpecifiedPoints

  :param data_container_array | DataContainerArray
  :param input_file_path | QString InputFilePath
  :param output_file_path | QString OutputFilePath
  :param observer | Observer
  """

  filter = sampling.SampleSurfaceMeshSpecifiedPoints()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if input_file_path is not None:
    filter.InputFilePath = input_file_path
  if output_file_path is not None:
    filter.OutputFilePath = output_file_path
  filter.execute()
  return filter.ErrorCode

def uncertain_regular_grid_sample_surface_mesh(data_container_array, data_container_name = None, cell_attribute_matrix_name = None, x_points = None, y_points = None, z_points = None, spacing = None, origin = None, uncertainty = None, feature_ids_array_name = None, observer = None) -> int:
  """
  UncertainRegularGridSampleSurfaceMesh

  :param data_container_array | DataContainerArray
  :param data_container_name | DataArrayPath DataContainerName
  :param cell_attribute_matrix_name | QString CellAttributeMatrixName
  :param x_points | int XPoints
  :param y_points | int YPoints
  :param z_points | int ZPoints
  :param spacing | FloatVec3Type Spacing
  :param origin | FloatVec3Type Origin
  :param uncertainty | FloatVec3Type Uncertainty
  :param feature_ids_array_name | QString FeatureIdsArrayName
  :param observer | Observer
  """

  filter = sampling.UncertainRegularGridSampleSurfaceMesh()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if data_container_name is not None:
    filter.DataContainerName = data_container_name
  if cell_attribute_matrix_name is not None:
    filter.CellAttributeMatrixName = cell_attribute_matrix_name
  if x_points is not None:
    filter.XPoints = x_points
  if y_points is not None:
    filter.YPoints = y_points
  if z_points is not None:
    filter.ZPoints = z_points
  if spacing is not None:
    filter.Spacing = spacing
  if origin is not None:
    filter.Origin = origin
  if uncertainty is not None:
    filter.Uncertainty = uncertainty
  if feature_ids_array_name is not None:
    filter.FeatureIdsArrayName = feature_ids_array_name
  filter.execute()
  return filter.ErrorCode

