from . import syntheticbuilding

def add_bad_data(data_container_array, gb_euclidean_distances_array_path = None, poisson_noise = None, poisson_vol_fraction = None, boundary_noise = None, boundary_vol_fraction = None, observer = None) -> int:
  """
  AddBadData

  :param data_container_array | DataContainerArray
  :param gb_euclidean_distances_array_path | DataArrayPath GBEuclideanDistancesArrayPath
  :param poisson_noise | bool PoissonNoise
  :param poisson_vol_fraction | float PoissonVolFraction
  :param boundary_noise | bool BoundaryNoise
  :param boundary_vol_fraction | float BoundaryVolFraction
  :param observer | Observer
  """

  filter = syntheticbuilding.AddBadData()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if gb_euclidean_distances_array_path is not None:
    filter.GBEuclideanDistancesArrayPath = gb_euclidean_distances_array_path
  if poisson_noise is not None:
    filter.PoissonNoise = poisson_noise
  if poisson_vol_fraction is not None:
    filter.PoissonVolFraction = poisson_vol_fraction
  if boundary_noise is not None:
    filter.BoundaryNoise = boundary_noise
  if boundary_vol_fraction is not None:
    filter.BoundaryVolFraction = boundary_vol_fraction
  filter.execute()
  return filter.ErrorCode

def add_orientation_noise(data_container_array, magnitude = None, cell_euler_angles_array_path = None, observer = None) -> int:
  """
  AddOrientationNoise

  :param data_container_array | DataContainerArray
  :param magnitude | float Magnitude
  :param cell_euler_angles_array_path | DataArrayPath CellEulerAnglesArrayPath
  :param observer | Observer
  """

  filter = syntheticbuilding.AddOrientationNoise()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if magnitude is not None:
    filter.Magnitude = magnitude
  if cell_euler_angles_array_path is not None:
    filter.CellEulerAnglesArrayPath = cell_euler_angles_array_path
  filter.execute()
  return filter.ErrorCode

def establish_matrix_phase(data_container_array, output_cell_attribute_matrix_path = None, feature_ids_array_name = None, cell_phases_array_name = None, feature_phases_array_name = None, num_features_array_name = None, mask_array_path = None, use_mask = None, input_stats_array_path = None, input_phase_types_array_path = None, input_phase_names_array_path = None, observer = None) -> int:
  """
  EstablishMatrixPhase

  :param data_container_array | DataContainerArray
  :param output_cell_attribute_matrix_path | DataArrayPath OutputCellAttributeMatrixPath
  :param feature_ids_array_name | QString FeatureIdsArrayName
  :param cell_phases_array_name | QString CellPhasesArrayName
  :param feature_phases_array_name | QString FeaturePhasesArrayName
  :param num_features_array_name | QString NumFeaturesArrayName
  :param mask_array_path | DataArrayPath MaskArrayPath
  :param use_mask | bool UseMask
  :param input_stats_array_path | DataArrayPath InputStatsArrayPath
  :param input_phase_types_array_path | DataArrayPath InputPhaseTypesArrayPath
  :param input_phase_names_array_path | DataArrayPath InputPhaseNamesArrayPath
  :param observer | Observer
  """

  filter = syntheticbuilding.EstablishMatrixPhase()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if output_cell_attribute_matrix_path is not None:
    filter.OutputCellAttributeMatrixPath = output_cell_attribute_matrix_path
  if feature_ids_array_name is not None:
    filter.FeatureIdsArrayName = feature_ids_array_name
  if cell_phases_array_name is not None:
    filter.CellPhasesArrayName = cell_phases_array_name
  if feature_phases_array_name is not None:
    filter.FeaturePhasesArrayName = feature_phases_array_name
  if num_features_array_name is not None:
    filter.NumFeaturesArrayName = num_features_array_name
  if mask_array_path is not None:
    filter.MaskArrayPath = mask_array_path
  if use_mask is not None:
    filter.UseMask = use_mask
  if input_stats_array_path is not None:
    filter.InputStatsArrayPath = input_stats_array_path
  if input_phase_types_array_path is not None:
    filter.InputPhaseTypesArrayPath = input_phase_types_array_path
  if input_phase_names_array_path is not None:
    filter.InputPhaseNamesArrayPath = input_phase_names_array_path
  filter.execute()
  return filter.ErrorCode

def establish_shape_types(data_container_array, input_phase_types_array_path = None, shape_types_array_name = None, shape_type_data = None, observer = None) -> int:
  """
  EstablishShapeTypes

  :param data_container_array | DataContainerArray
  :param input_phase_types_array_path | DataArrayPath InputPhaseTypesArrayPath
  :param shape_types_array_name | QString ShapeTypesArrayName
  :param shape_type_data | ShapeType::Types ShapeTypeData
  :param observer | Observer
  """

  filter = syntheticbuilding.EstablishShapeTypes()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if input_phase_types_array_path is not None:
    filter.InputPhaseTypesArrayPath = input_phase_types_array_path
  if shape_types_array_name is not None:
    filter.ShapeTypesArrayName = shape_types_array_name
  if shape_type_data is not None:
    filter.ShapeTypeData = shape_type_data
  filter.execute()
  return filter.ErrorCode

def initialize_synthetic_volume(data_container_array, data_container_name = None, cell_attribute_matrix_name = None, ensemble_attribute_matrix_name = None, length_unit = None, dimensions = None, spacing = None, origin = None, input_stats_array_path = None, input_phase_types_array_path = None, input_phase_names_array_path = None, estimate_number_of_features = None, geometry_selection = None, geometry_data_container = None, observer = None) -> int:
  """
  InitializeSyntheticVolume

  :param data_container_array | DataContainerArray
  :param data_container_name | DataArrayPath DataContainerName
  :param cell_attribute_matrix_name | QString CellAttributeMatrixName
  :param ensemble_attribute_matrix_name | QString EnsembleAttributeMatrixName
  :param length_unit | int32_t LengthUnit
  :param dimensions | IntVec3Type Dimensions
  :param spacing | FloatVec3Type Spacing
  :param origin | FloatVec3Type Origin
  :param input_stats_array_path | DataArrayPath InputStatsArrayPath
  :param input_phase_types_array_path | DataArrayPath InputPhaseTypesArrayPath
  :param input_phase_names_array_path | DataArrayPath InputPhaseNamesArrayPath
  :param estimate_number_of_features | bool EstimateNumberOfFeatures
  :param geometry_selection | int32_t GeometrySelection
  :param geometry_data_container | DataArrayPath GeometryDataContainer
  :param observer | Observer
  """

  filter = syntheticbuilding.InitializeSyntheticVolume()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if data_container_name is not None:
    filter.DataContainerName = data_container_name
  if cell_attribute_matrix_name is not None:
    filter.CellAttributeMatrixName = cell_attribute_matrix_name
  if ensemble_attribute_matrix_name is not None:
    filter.EnsembleAttributeMatrixName = ensemble_attribute_matrix_name
  if length_unit is not None:
    filter.LengthUnit = length_unit
  if dimensions is not None:
    filter.Dimensions = dimensions
  if spacing is not None:
    filter.Spacing = spacing
  if origin is not None:
    filter.Origin = origin
  if input_stats_array_path is not None:
    filter.InputStatsArrayPath = input_stats_array_path
  if input_phase_types_array_path is not None:
    filter.InputPhaseTypesArrayPath = input_phase_types_array_path
  if input_phase_names_array_path is not None:
    filter.InputPhaseNamesArrayPath = input_phase_names_array_path
  if estimate_number_of_features is not None:
    filter.EstimateNumberOfFeatures = estimate_number_of_features
  if geometry_selection is not None:
    filter.GeometrySelection = geometry_selection
  if geometry_data_container is not None:
    filter.GeometryDataContainer = geometry_data_container
  filter.execute()
  return filter.ErrorCode

def insert_atoms(data_container_array, vertex_data_container_name = None, vertex_attribute_matrix_name = None, lattice_constants = None, basis = None, surface_mesh_face_labels_array_path = None, avg_quats_array_path = None, atom_feature_labels_array_name = None, observer = None) -> int:
  """
  InsertAtoms

  :param data_container_array | DataContainerArray
  :param vertex_data_container_name | DataArrayPath VertexDataContainerName
  :param vertex_attribute_matrix_name | QString VertexAttributeMatrixName
  :param lattice_constants | FloatVec3Type LatticeConstants
  :param basis | int Basis
  :param surface_mesh_face_labels_array_path | DataArrayPath SurfaceMeshFaceLabelsArrayPath
  :param avg_quats_array_path | DataArrayPath AvgQuatsArrayPath
  :param atom_feature_labels_array_name | QString AtomFeatureLabelsArrayName
  :param observer | Observer
  """

  filter = syntheticbuilding.InsertAtoms()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if vertex_data_container_name is not None:
    filter.VertexDataContainerName = vertex_data_container_name
  if vertex_attribute_matrix_name is not None:
    filter.VertexAttributeMatrixName = vertex_attribute_matrix_name
  if lattice_constants is not None:
    filter.LatticeConstants = lattice_constants
  if basis is not None:
    filter.Basis = basis
  if surface_mesh_face_labels_array_path is not None:
    filter.SurfaceMeshFaceLabelsArrayPath = surface_mesh_face_labels_array_path
  if avg_quats_array_path is not None:
    filter.AvgQuatsArrayPath = avg_quats_array_path
  if atom_feature_labels_array_name is not None:
    filter.AtomFeatureLabelsArrayName = atom_feature_labels_array_name
  filter.execute()
  return filter.ErrorCode

def insert_precipitate_phases(data_container_array, csv_output_file = None, mask_array_path = None, use_mask = None, feature_generation = None, precip_input_file = None, periodic_boundaries = None, match_rdf = None, write_goal_attributes = None, input_stats_array_path = None, input_phase_types_array_path = None, input_shape_types_array_path = None, feature_ids_array_path = None, cell_phases_array_path = None, boundary_cells_array_path = None, feature_phases_array_path = None, num_cells_array_name = None, equivalent_diameters_array_name = None, volumes_array_name = None, omega3s_array_name = None, centroids_array_name = None, axis_euler_angles_array_name = None, axis_lengths_array_name = None, num_features_array_path = None, save_geometric_descriptions = None, new_attribute_matrix_path = None, selected_attribute_matrix_path = None, observer = None) -> int:
  """
  InsertPrecipitatePhases

  :param data_container_array | DataContainerArray
  :param csv_output_file | QString CsvOutputFile
  :param mask_array_path | DataArrayPath MaskArrayPath
  :param use_mask | bool UseMask
  :param feature_generation | int FeatureGeneration
  :param precip_input_file | QString PrecipInputFile
  :param periodic_boundaries | bool PeriodicBoundaries
  :param match_rdf | bool MatchRDF
  :param write_goal_attributes | bool WriteGoalAttributes
  :param input_stats_array_path | DataArrayPath InputStatsArrayPath
  :param input_phase_types_array_path | DataArrayPath InputPhaseTypesArrayPath
  :param input_shape_types_array_path | DataArrayPath InputShapeTypesArrayPath
  :param feature_ids_array_path | DataArrayPath FeatureIdsArrayPath
  :param cell_phases_array_path | DataArrayPath CellPhasesArrayPath
  :param boundary_cells_array_path | DataArrayPath BoundaryCellsArrayPath
  :param feature_phases_array_path | DataArrayPath FeaturePhasesArrayPath
  :param num_cells_array_name | QString NumCellsArrayName
  :param equivalent_diameters_array_name | QString EquivalentDiametersArrayName
  :param volumes_array_name | QString VolumesArrayName
  :param omega3s_array_name | QString Omega3sArrayName
  :param centroids_array_name | QString CentroidsArrayName
  :param axis_euler_angles_array_name | QString AxisEulerAnglesArrayName
  :param axis_lengths_array_name | QString AxisLengthsArrayName
  :param num_features_array_path | DataArrayPath NumFeaturesArrayPath
  :param save_geometric_descriptions | int SaveGeometricDescriptions
  :param new_attribute_matrix_path | DataArrayPath NewAttributeMatrixPath
  :param selected_attribute_matrix_path | DataArrayPath SelectedAttributeMatrixPath
  :param observer | Observer
  """

  filter = syntheticbuilding.InsertPrecipitatePhases()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if csv_output_file is not None:
    filter.CsvOutputFile = csv_output_file
  if mask_array_path is not None:
    filter.MaskArrayPath = mask_array_path
  if use_mask is not None:
    filter.UseMask = use_mask
  if feature_generation is not None:
    filter.FeatureGeneration = feature_generation
  if precip_input_file is not None:
    filter.PrecipInputFile = precip_input_file
  if periodic_boundaries is not None:
    filter.PeriodicBoundaries = periodic_boundaries
  if match_rdf is not None:
    filter.MatchRDF = match_rdf
  if write_goal_attributes is not None:
    filter.WriteGoalAttributes = write_goal_attributes
  if input_stats_array_path is not None:
    filter.InputStatsArrayPath = input_stats_array_path
  if input_phase_types_array_path is not None:
    filter.InputPhaseTypesArrayPath = input_phase_types_array_path
  if input_shape_types_array_path is not None:
    filter.InputShapeTypesArrayPath = input_shape_types_array_path
  if feature_ids_array_path is not None:
    filter.FeatureIdsArrayPath = feature_ids_array_path
  if cell_phases_array_path is not None:
    filter.CellPhasesArrayPath = cell_phases_array_path
  if boundary_cells_array_path is not None:
    filter.BoundaryCellsArrayPath = boundary_cells_array_path
  if feature_phases_array_path is not None:
    filter.FeaturePhasesArrayPath = feature_phases_array_path
  if num_cells_array_name is not None:
    filter.NumCellsArrayName = num_cells_array_name
  if equivalent_diameters_array_name is not None:
    filter.EquivalentDiametersArrayName = equivalent_diameters_array_name
  if volumes_array_name is not None:
    filter.VolumesArrayName = volumes_array_name
  if omega3s_array_name is not None:
    filter.Omega3sArrayName = omega3s_array_name
  if centroids_array_name is not None:
    filter.CentroidsArrayName = centroids_array_name
  if axis_euler_angles_array_name is not None:
    filter.AxisEulerAnglesArrayName = axis_euler_angles_array_name
  if axis_lengths_array_name is not None:
    filter.AxisLengthsArrayName = axis_lengths_array_name
  if num_features_array_path is not None:
    filter.NumFeaturesArrayPath = num_features_array_path
  if save_geometric_descriptions is not None:
    filter.SaveGeometricDescriptions = save_geometric_descriptions
  if new_attribute_matrix_path is not None:
    filter.NewAttributeMatrixPath = new_attribute_matrix_path
  if selected_attribute_matrix_path is not None:
    filter.SelectedAttributeMatrixPath = selected_attribute_matrix_path
  filter.execute()
  return filter.ErrorCode

def jumble_orientations(data_container_array, feature_ids_array_path = None, cell_euler_angles_array_name = None, feature_phases_array_path = None, feature_euler_angles_array_path = None, avg_quats_array_name = None, observer = None) -> int:
  """
  JumbleOrientations

  :param data_container_array | DataContainerArray
  :param feature_ids_array_path | DataArrayPath FeatureIdsArrayPath
  :param cell_euler_angles_array_name | QString CellEulerAnglesArrayName
  :param feature_phases_array_path | DataArrayPath FeaturePhasesArrayPath
  :param feature_euler_angles_array_path | DataArrayPath FeatureEulerAnglesArrayPath
  :param avg_quats_array_name | QString AvgQuatsArrayName
  :param observer | Observer
  """

  filter = syntheticbuilding.JumbleOrientations()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if feature_ids_array_path is not None:
    filter.FeatureIdsArrayPath = feature_ids_array_path
  if cell_euler_angles_array_name is not None:
    filter.CellEulerAnglesArrayName = cell_euler_angles_array_name
  if feature_phases_array_path is not None:
    filter.FeaturePhasesArrayPath = feature_phases_array_path
  if feature_euler_angles_array_path is not None:
    filter.FeatureEulerAnglesArrayPath = feature_euler_angles_array_path
  if avg_quats_array_name is not None:
    filter.AvgQuatsArrayName = avg_quats_array_name
  filter.execute()
  return filter.ErrorCode

def match_crystallography(data_container_array, input_stats_array_path = None, crystal_structures_array_path = None, phase_types_array_path = None, feature_ids_array_path = None, feature_phases_array_path = None, surface_features_array_path = None, neighbor_list_array_path = None, shared_surface_area_list_array_path = None, num_features_array_path = None, cell_euler_angles_array_name = None, volumes_array_name = None, feature_euler_angles_array_name = None, avg_quats_array_name = None, max_iterations = None, observer = None) -> int:
  """
  MatchCrystallography

  :param data_container_array | DataContainerArray
  :param input_stats_array_path | DataArrayPath InputStatsArrayPath
  :param crystal_structures_array_path | DataArrayPath CrystalStructuresArrayPath
  :param phase_types_array_path | DataArrayPath PhaseTypesArrayPath
  :param feature_ids_array_path | DataArrayPath FeatureIdsArrayPath
  :param feature_phases_array_path | DataArrayPath FeaturePhasesArrayPath
  :param surface_features_array_path | DataArrayPath SurfaceFeaturesArrayPath
  :param neighbor_list_array_path | DataArrayPath NeighborListArrayPath
  :param shared_surface_area_list_array_path | DataArrayPath SharedSurfaceAreaListArrayPath
  :param num_features_array_path | DataArrayPath NumFeaturesArrayPath
  :param cell_euler_angles_array_name | QString CellEulerAnglesArrayName
  :param volumes_array_name | QString VolumesArrayName
  :param feature_euler_angles_array_name | QString FeatureEulerAnglesArrayName
  :param avg_quats_array_name | QString AvgQuatsArrayName
  :param max_iterations | int MaxIterations
  :param observer | Observer
  """

  filter = syntheticbuilding.MatchCrystallography()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if input_stats_array_path is not None:
    filter.InputStatsArrayPath = input_stats_array_path
  if crystal_structures_array_path is not None:
    filter.CrystalStructuresArrayPath = crystal_structures_array_path
  if phase_types_array_path is not None:
    filter.PhaseTypesArrayPath = phase_types_array_path
  if feature_ids_array_path is not None:
    filter.FeatureIdsArrayPath = feature_ids_array_path
  if feature_phases_array_path is not None:
    filter.FeaturePhasesArrayPath = feature_phases_array_path
  if surface_features_array_path is not None:
    filter.SurfaceFeaturesArrayPath = surface_features_array_path
  if neighbor_list_array_path is not None:
    filter.NeighborListArrayPath = neighbor_list_array_path
  if shared_surface_area_list_array_path is not None:
    filter.SharedSurfaceAreaListArrayPath = shared_surface_area_list_array_path
  if num_features_array_path is not None:
    filter.NumFeaturesArrayPath = num_features_array_path
  if cell_euler_angles_array_name is not None:
    filter.CellEulerAnglesArrayName = cell_euler_angles_array_name
  if volumes_array_name is not None:
    filter.VolumesArrayName = volumes_array_name
  if feature_euler_angles_array_name is not None:
    filter.FeatureEulerAnglesArrayName = feature_euler_angles_array_name
  if avg_quats_array_name is not None:
    filter.AvgQuatsArrayName = avg_quats_array_name
  if max_iterations is not None:
    filter.MaxIterations = max_iterations
  filter.execute()
  return filter.ErrorCode

def pack_primary_phases(data_container_array, output_cell_attribute_matrix_path = None, output_cell_feature_attribute_matrix_name = None, output_cell_ensemble_attribute_matrix_name = None, feature_ids_array_name = None, cell_phases_array_name = None, feature_phases_array_name = None, num_features_array_name = None, input_stats_array_path = None, input_phase_types_array_path = None, input_phase_names_array_path = None, input_shape_types_array_path = None, mask_array_path = None, use_mask = None, feature_generation = None, feature_input_file = None, csv_output_file = None, periodic_boundaries = None, write_goal_attributes = None, save_geometric_descriptions = None, new_attribute_matrix_path = None, selected_attribute_matrix_path = None, observer = None) -> int:
  """
  PackPrimaryPhases

  :param data_container_array | DataContainerArray
  :param output_cell_attribute_matrix_path | DataArrayPath OutputCellAttributeMatrixPath
  :param output_cell_feature_attribute_matrix_name | QString OutputCellFeatureAttributeMatrixName
  :param output_cell_ensemble_attribute_matrix_name | QString OutputCellEnsembleAttributeMatrixName
  :param feature_ids_array_name | QString FeatureIdsArrayName
  :param cell_phases_array_name | QString CellPhasesArrayName
  :param feature_phases_array_name | QString FeaturePhasesArrayName
  :param num_features_array_name | QString NumFeaturesArrayName
  :param input_stats_array_path | DataArrayPath InputStatsArrayPath
  :param input_phase_types_array_path | DataArrayPath InputPhaseTypesArrayPath
  :param input_phase_names_array_path | DataArrayPath InputPhaseNamesArrayPath
  :param input_shape_types_array_path | DataArrayPath InputShapeTypesArrayPath
  :param mask_array_path | DataArrayPath MaskArrayPath
  :param use_mask | bool UseMask
  :param feature_generation | int FeatureGeneration
  :param feature_input_file | QString FeatureInputFile
  :param csv_output_file | QString CsvOutputFile
  :param periodic_boundaries | bool PeriodicBoundaries
  :param write_goal_attributes | bool WriteGoalAttributes
  :param save_geometric_descriptions | int SaveGeometricDescriptions
  :param new_attribute_matrix_path | DataArrayPath NewAttributeMatrixPath
  :param selected_attribute_matrix_path | DataArrayPath SelectedAttributeMatrixPath
  :param observer | Observer
  """

  filter = syntheticbuilding.PackPrimaryPhases()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if output_cell_attribute_matrix_path is not None:
    filter.OutputCellAttributeMatrixPath = output_cell_attribute_matrix_path
  if output_cell_feature_attribute_matrix_name is not None:
    filter.OutputCellFeatureAttributeMatrixName = output_cell_feature_attribute_matrix_name
  if output_cell_ensemble_attribute_matrix_name is not None:
    filter.OutputCellEnsembleAttributeMatrixName = output_cell_ensemble_attribute_matrix_name
  if feature_ids_array_name is not None:
    filter.FeatureIdsArrayName = feature_ids_array_name
  if cell_phases_array_name is not None:
    filter.CellPhasesArrayName = cell_phases_array_name
  if feature_phases_array_name is not None:
    filter.FeaturePhasesArrayName = feature_phases_array_name
  if num_features_array_name is not None:
    filter.NumFeaturesArrayName = num_features_array_name
  if input_stats_array_path is not None:
    filter.InputStatsArrayPath = input_stats_array_path
  if input_phase_types_array_path is not None:
    filter.InputPhaseTypesArrayPath = input_phase_types_array_path
  if input_phase_names_array_path is not None:
    filter.InputPhaseNamesArrayPath = input_phase_names_array_path
  if input_shape_types_array_path is not None:
    filter.InputShapeTypesArrayPath = input_shape_types_array_path
  if mask_array_path is not None:
    filter.MaskArrayPath = mask_array_path
  if use_mask is not None:
    filter.UseMask = use_mask
  if feature_generation is not None:
    filter.FeatureGeneration = feature_generation
  if feature_input_file is not None:
    filter.FeatureInputFile = feature_input_file
  if csv_output_file is not None:
    filter.CsvOutputFile = csv_output_file
  if periodic_boundaries is not None:
    filter.PeriodicBoundaries = periodic_boundaries
  if write_goal_attributes is not None:
    filter.WriteGoalAttributes = write_goal_attributes
  if save_geometric_descriptions is not None:
    filter.SaveGeometricDescriptions = save_geometric_descriptions
  if new_attribute_matrix_path is not None:
    filter.NewAttributeMatrixPath = new_attribute_matrix_path
  if selected_attribute_matrix_path is not None:
    filter.SelectedAttributeMatrixPath = selected_attribute_matrix_path
  filter.execute()
  return filter.ErrorCode

def stats_generator_filter(data_container_array, stats_generator_data_container_name = None, cell_ensemble_attribute_matrix_name = None, stats_data_array_name = None, crystal_structures_array_name = None, phase_types_array_name = None, phase_names_array_name = None, stats_data_array = None, crystal_structures = None, phase_types = None, phase_names = None, path = None, observer = None) -> int:
  """
  StatsGeneratorFilter

  :param data_container_array | DataContainerArray
  :param stats_generator_data_container_name | DataArrayPath StatsGeneratorDataContainerName
  :param cell_ensemble_attribute_matrix_name | QString CellEnsembleAttributeMatrixName
  :param stats_data_array_name | QString StatsDataArrayName
  :param crystal_structures_array_name | QString CrystalStructuresArrayName
  :param phase_types_array_name | QString PhaseTypesArrayName
  :param phase_names_array_name | QString PhaseNamesArrayName
  :param stats_data_array | StatsDataArray::Pointer StatsDataArray
  :param crystal_structures | UInt32ArrayType::Pointer CrystalStructures
  :param phase_types | UInt32ArrayType::Pointer PhaseTypes
  :param phase_names | StringDataArray::Pointer PhaseNames
  :param path | DataArrayPath Path
  :param observer | Observer
  """

  filter = syntheticbuilding.StatsGeneratorFilter()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if stats_generator_data_container_name is not None:
    filter.StatsGeneratorDataContainerName = stats_generator_data_container_name
  if cell_ensemble_attribute_matrix_name is not None:
    filter.CellEnsembleAttributeMatrixName = cell_ensemble_attribute_matrix_name
  if stats_data_array_name is not None:
    filter.StatsDataArrayName = stats_data_array_name
  if crystal_structures_array_name is not None:
    filter.CrystalStructuresArrayName = crystal_structures_array_name
  if phase_types_array_name is not None:
    filter.PhaseTypesArrayName = phase_types_array_name
  if phase_names_array_name is not None:
    filter.PhaseNamesArrayName = phase_names_array_name
  if stats_data_array is not None:
    filter.StatsDataArray = stats_data_array
  if crystal_structures is not None:
    filter.CrystalStructures = crystal_structures
  if phase_types is not None:
    filter.PhaseTypes = phase_types
  if phase_names is not None:
    filter.PhaseNames = phase_names
  if path is not None:
    filter.Path = path
  filter.execute()
  return filter.ErrorCode

def generate_primary_stats_data(data_container_array, phase_name = None, phase_index = None, crystal_symmetry = None, micro_preset_model = None, phase_fraction = None, mu = None, sigma = None, min_cut_off = None, max_cut_off = None, bin_step_size = None, create_ensemble_attribute_matrix = None, data_container_name = None, cell_ensemble_attribute_matrix_name = None, append_to_existing_attribute_matrix = None, selected_ensemble_attribute_matrix = None, odf_data = None, mdf_data = None, axis_odf_data = None, observer = None) -> int:
  """
  GeneratePrimaryStatsData

  :param data_container_array | DataContainerArray
  :param phase_name | QString PhaseName
  :param phase_index | int PhaseIndex
  :param crystal_symmetry | int CrystalSymmetry
  :param micro_preset_model | int MicroPresetModel
  :param phase_fraction | double PhaseFraction
  :param mu | double Mu
  :param sigma | double Sigma
  :param min_cut_off | double MinCutOff
  :param max_cut_off | double MaxCutOff
  :param bin_step_size | double BinStepSize
  :param create_ensemble_attribute_matrix | bool CreateEnsembleAttributeMatrix
  :param data_container_name | DataArrayPath DataContainerName
  :param cell_ensemble_attribute_matrix_name | QString CellEnsembleAttributeMatrixName
  :param append_to_existing_attribute_matrix | bool AppendToExistingAttributeMatrix
  :param selected_ensemble_attribute_matrix | DataArrayPath SelectedEnsembleAttributeMatrix
  :param odf_data | DynamicTableData OdfData
  :param mdf_data | DynamicTableData MdfData
  :param axis_odf_data | DynamicTableData AxisOdfData
  :param observer | Observer
  """

  filter = syntheticbuilding.GeneratePrimaryStatsData()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if phase_name is not None:
    filter.PhaseName = phase_name
  if phase_index is not None:
    filter.PhaseIndex = phase_index
  if crystal_symmetry is not None:
    filter.CrystalSymmetry = crystal_symmetry
  if micro_preset_model is not None:
    filter.MicroPresetModel = micro_preset_model
  if phase_fraction is not None:
    filter.PhaseFraction = phase_fraction
  if mu is not None:
    filter.Mu = mu
  if sigma is not None:
    filter.Sigma = sigma
  if min_cut_off is not None:
    filter.MinCutOff = min_cut_off
  if max_cut_off is not None:
    filter.MaxCutOff = max_cut_off
  if bin_step_size is not None:
    filter.BinStepSize = bin_step_size
  if create_ensemble_attribute_matrix is not None:
    filter.CreateEnsembleAttributeMatrix = create_ensemble_attribute_matrix
  if data_container_name is not None:
    filter.DataContainerName = data_container_name
  if cell_ensemble_attribute_matrix_name is not None:
    filter.CellEnsembleAttributeMatrixName = cell_ensemble_attribute_matrix_name
  if append_to_existing_attribute_matrix is not None:
    filter.AppendToExistingAttributeMatrix = append_to_existing_attribute_matrix
  if selected_ensemble_attribute_matrix is not None:
    filter.SelectedEnsembleAttributeMatrix = selected_ensemble_attribute_matrix
  if odf_data is not None:
    filter.OdfData = odf_data
  if mdf_data is not None:
    filter.MdfData = mdf_data
  if axis_odf_data is not None:
    filter.AxisOdfData = axis_odf_data
  filter.execute()
  return filter.ErrorCode

def generate_precipitate_stats_data(data_container_array, phase_name = None, phase_index = None, crystal_symmetry = None, micro_preset_model = None, phase_fraction = None, mu = None, sigma = None, min_cut_off = None, max_cut_off = None, bin_step_size = None, create_ensemble_attribute_matrix = None, data_container_name = None, cell_ensemble_attribute_matrix_name = None, append_to_existing_attribute_matrix = None, selected_ensemble_attribute_matrix = None, odf_data = None, mdf_data = None, axis_odf_data = None, rdf_min_max_distance = None, rdf_num_bins = None, rdf_box_size = None, observer = None) -> int:
  """
  GeneratePrecipitateStatsData

  :param data_container_array | DataContainerArray
  :param phase_name | QString PhaseName
  :param phase_index | int PhaseIndex
  :param crystal_symmetry | int CrystalSymmetry
  :param micro_preset_model | int MicroPresetModel
  :param phase_fraction | double PhaseFraction
  :param mu | double Mu
  :param sigma | double Sigma
  :param min_cut_off | double MinCutOff
  :param max_cut_off | double MaxCutOff
  :param bin_step_size | double BinStepSize
  :param create_ensemble_attribute_matrix | bool CreateEnsembleAttributeMatrix
  :param data_container_name | DataArrayPath DataContainerName
  :param cell_ensemble_attribute_matrix_name | QString CellEnsembleAttributeMatrixName
  :param append_to_existing_attribute_matrix | bool AppendToExistingAttributeMatrix
  :param selected_ensemble_attribute_matrix | DataArrayPath SelectedEnsembleAttributeMatrix
  :param odf_data | DynamicTableData OdfData
  :param mdf_data | DynamicTableData MdfData
  :param axis_odf_data | DynamicTableData AxisOdfData
  :param rdf_min_max_distance | FloatVec2Type RdfMinMaxDistance
  :param rdf_num_bins | int RdfNumBins
  :param rdf_box_size | FloatVec3Type RdfBoxSize
  :param observer | Observer
  """

  filter = syntheticbuilding.GeneratePrecipitateStatsData()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if phase_name is not None:
    filter.PhaseName = phase_name
  if phase_index is not None:
    filter.PhaseIndex = phase_index
  if crystal_symmetry is not None:
    filter.CrystalSymmetry = crystal_symmetry
  if micro_preset_model is not None:
    filter.MicroPresetModel = micro_preset_model
  if phase_fraction is not None:
    filter.PhaseFraction = phase_fraction
  if mu is not None:
    filter.Mu = mu
  if sigma is not None:
    filter.Sigma = sigma
  if min_cut_off is not None:
    filter.MinCutOff = min_cut_off
  if max_cut_off is not None:
    filter.MaxCutOff = max_cut_off
  if bin_step_size is not None:
    filter.BinStepSize = bin_step_size
  if create_ensemble_attribute_matrix is not None:
    filter.CreateEnsembleAttributeMatrix = create_ensemble_attribute_matrix
  if data_container_name is not None:
    filter.DataContainerName = data_container_name
  if cell_ensemble_attribute_matrix_name is not None:
    filter.CellEnsembleAttributeMatrixName = cell_ensemble_attribute_matrix_name
  if append_to_existing_attribute_matrix is not None:
    filter.AppendToExistingAttributeMatrix = append_to_existing_attribute_matrix
  if selected_ensemble_attribute_matrix is not None:
    filter.SelectedEnsembleAttributeMatrix = selected_ensemble_attribute_matrix
  if odf_data is not None:
    filter.OdfData = odf_data
  if mdf_data is not None:
    filter.MdfData = mdf_data
  if axis_odf_data is not None:
    filter.AxisOdfData = axis_odf_data
  if rdf_min_max_distance is not None:
    filter.RdfMinMaxDistance = rdf_min_max_distance
  if rdf_num_bins is not None:
    filter.RdfNumBins = rdf_num_bins
  if rdf_box_size is not None:
    filter.RdfBoxSize = rdf_box_size
  filter.execute()
  return filter.ErrorCode

